@@ -9,7 +9,7 @@ use Carp qw(croak);
 use Sub::Accessor::Small ();
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.014';
+our $VERSION   = '1.000000';
 our @EXPORT    = qw/ lexical_has /;
 our @ISA       = qw/ Sub::Accessor::Small /;
 
@@ -17,9 +17,7 @@ sub _generate_lexical_has : method
 {
 	my $me = shift;
 	my $code = $me->_generate_has(@_);
-	$code = Sub::Name::subname("$me\::lexical_has", $code)
-		if Sub::Accessor::Small::HAS_SUB_NAME;
-	return $code;
+	return Sub::Accessor::Small::set_subname( "$me\::lexical_has", $code );
 }
 
 sub lexical_has : method
@@ -33,8 +31,8 @@ sub inline_to_coderef : method
 	my $me = shift;
 	my ($method_type, $code) = @_;
 	my $coderef = $me->SUPER::inline_to_coderef(@_);
-	Sub::Accessor::Small::HAS_SUB_NAME && $me->{package} && defined($me->{slot})
-		? Sub::Name::subname("$me->{package}\::__LEXICAL__[$me->{slot}]", $coderef)
+	( $me->{package} and defined $me->{slot} )
+		? Sub::Accessor::Small::set_subname( "$me->{package}\::__LEXICAL__[$me->{slot}]", $coderef )
 		: $coderef
 }
 
@@ -402,7 +400,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017, 2025 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
